--------------------------------------------
-- Array Randomize                           --
-- Copyright by Anders K. Nielsen         --
-- All rights reserved.                   --
-- URL: http://www.mcgreed.dk             --
-- Contact: akn@mcgreed.dk                --
-- v 0.83                                 --
--------------------------------------------

plugin modifier Array3DRandomizeMod
name:"Array3D Randomize"
classID:#(1164269483, -134014096)
version:1
-- interal version to public: 1: 0.83
(
-- 	parameters mainSourceArray
-- 	(
-- 		dataArrayList type:#integer animatable:false
-- 		intUniqueArrayID type:#integer animatable:false
-- 	)
	parameters mainRandomize rollout:rolloutRandomize
	(
		boolRandomizeOn type:#boolean animatable:false ui:boolRandomizeOn checked:false enabled:true
		intRanSeed type:#integer animatable:false ui:intRanSeed default:1

		floatRanPosX type:#worldUnits animatable:false ui:floatRanPosX default:0
		boolRanPosBothX type:#boolean animatable:false ui:boolRanPosBothX checked:false
		floatRanPosStepX type:#float animatable:false ui:floatRanPosStepX default:0.01
		floatRanPosY type:#worldUnits animatable:false ui:floatRanPosY default:0
		boolRanPosBothY type:#boolean animatable:false ui:boolRanPosBothY checked:false
		floatRanPosStepY type:#float animatable:false ui:floatRanPosStepY default:0.01
		floatRanPosZ type:#worldUnits animatable:false ui:floatRanPosZ default:0
		boolRanPosBothZ type:#boolean animatable:false ui:boolRanPosBothZ checked:false
		floatRanPosStepZ type:#float animatable:false ui:floatRanPosStepZ default:0.01
				
		floatRanRotX type:#float animatable:false ui:floatRanRotX default:0
		boolRanRotBothX type:#boolean animatable:false ui:boolRanRotBothX checked:false
		floatRanRotStepX type:#float animatable:false ui:floatRanRotStepX default:0.01
		floatRanRotY type:#float animatable:false ui:floatRanRotY default:0
		boolRanRotBothY type:#boolean animatable:false ui:boolRanRotBothY checked:false
		floatRanRotStepY type:#float animatable:false ui:floatRanRotStepY default:0.01
		floatRanRotZ type:#float animatable:false ui:floatRanRotZ default:0
		boolRanRotBothZ type:#boolean animatable:false ui:boolRanRotBothZ checked:false
		floatRanRotStepZ type:#float animatable:false ui:floatRanRotStepZ default:0.01
		
		floatRanScaX type:#float animatable:false ui:floatRanScaX default:0
		boolRanScaBothX type:#boolean animatable:false ui:boolRanScaBothX checked:false
		floatRanScaStepX type:#float animatable:false ui:floatRanScaStepX default:0.01
		floatRanScaY type:#float animatable:false ui:floatRanScaY default:0
		boolRanScaBothY type:#boolean animatable:false ui:boolRanScaBothY checked:false
		floatRanScaStepY type:#float animatable:false ui:floatRanScaStepY default:0.01
		floatRanScaZ type:#float animatable:false ui:floatRanScaZ default:0
		boolRanScaBothZ type:#boolean animatable:false ui:boolRanScaBothZ checked:false
		floatRanScaStepZ type:#float animatable:false ui:floatRanScaStepZ default:0.01
	)
		
	local rolloutSourceArray
	local dataArray
	local pathDir = (GetDir #scripts) + "\\Array3D\\"

	fn SetObjectRotation obj rx ry rz =
	(
		local translateMat = transMatrix obj.transform.pos
		local scaleMat = scaleMatrix obj.transform.scale
		obj.transform = scaleMat * translateMat

		-- Perform each axis rotation individually
		rotate obj (angleaxis rx [1,0,0])
		rotate obj (angleaxis ry [0,1,0])
		rotate obj (angleaxis rz [0,0,1])
	)
		
	fn updateArrayData arrayID  =
	(
		selectedSourceID = arrayID
	
		if arrayID == 1 then (
			intUniqueArrayID = 0
			rolloutSourceArray.lblSourceArrayID.text = "Array ID: *NOT SELECTED*"
			messagebox "You need to pick an array in the dropdown box for randomize to work."
		) else (
			-- Collect all the Array 3D modifiers on the object, and put them in an array
			intUniqueArrayID = dataArray[(arrayID-1)].intUniqueArrayID
			rolloutSourceArray.lblSourceArrayID.text = "Array ID: #" + intUniqueArrayID as string
		)
		if dataArray.count != 0 and intUniqueArrayID != 0 do
		(
			for x in dataArray where x.intUniqueArrayID == intUniqueArrayID do
			(
				x.updateArrayObj 2 1 1 1 x.intAAmount1D x.intAAmount2D x.intAAmount3D 1 1 1 1 1 1 1
				--x.updateArrayObj 2 1 1 1 x.intAAmount1D x.intAAmount2D x.intAAmount3D 1 1 1 1 changePos changeRot changeSca
			)
		)
	)
	
	fn updateRandom changePos changeRot changeSca =
	(
		if floatRanPosStepX <= 0 do floatRanPosStepX = 1
		if floatRanPosStepY <= 0 do floatRanPosStepY = 1
		if floatRanPosStepZ <= 0 do floatRanPosStepZ = 1
		if floatRanRotStepX <= 0 do floatRanRotStepX = 1
		if floatRanRotStepY <= 0 do floatRanRotStepY = 1
		if floatRanRotStepZ <= 0 do floatRanRotStepZ = 1
		if floatRanScaStepX <= 0 do floatRanScaStepX = 1
		if floatRanScaStepY <= 0 do floatRanScaStepY = 1
		if floatRanScaStepZ <= 0 do floatRanScaStepZ = 1

		$.updateArrayObj 2 1 1 1 $.intAAmount1D $.intAAmount2D $.intAAmount3D 1 1 1 1 1 1 1
	)

	rollout rolloutRandomize "Randomize" width:162 height:297
	(
		checkbox boolRandomizeOn "Activate Randomize?" pos:[7,4] width:151 height:15
		spinner intRanSeed "Seed: " type:#integer pos:[15,24] width:110 height:16 range:[1,999999,1] scale:1
		button btnGenerateSeed "G" pos:[133,23] width:18 height:18
		
		label lblPosition "Position" pos:[28,47] width:40 height:14
		label lblStepPos "Step" pos:[100,47] width:23 height:14
		spinner floatRanPosX "X: " pos:[15,61] width:74 height:16 range:[-100000,100000,0] scale:0.1
		spinner floatRanPosStepX "" pos:[93,61] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanPosBothX "" pos:[142,62] width:14 height:14
		spinner floatRanPosY "Y: " pos:[15,80] width:74 height:16 range:[-100000,100000,0] scale:0.1
		spinner floatRanPosStepY "" pos:[93,80] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanPosBothY "" pos:[142,81] width:14 height:14
		spinner floatRanPosZ "Z: " pos:[15,99] width:74 height:16 range:[-100000,100000,0] scale:0.1
		spinner floatRanPosStepZ "" pos:[93,99] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanPosBothZ "" pos:[142,100] width:14 height:14
		
		ImgTag bmpBothWay1 "" pos:[142,119] width:13 height:14 bitmap:(openBitMap (pathDir + "array_icon_bothway.png")) transparent:white
		
		label lblStepRot "Step" pos:[100,122] width:23 height:14
		label lblRotation "Rotation" pos:[28,122] width:40 height:14
		spinner floatRanRotX "X: " pos:[15,136] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanRotStepX "" pos:[93,136] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanRotBothX "" pos:[142,137] width:14 height:14
		spinner floatRanRotY "Y: " pos:[15,155] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanRotStepY "" pos:[93,155] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanRotBothY "" pos:[142,156] width:14 height:14
		spinner floatRanRotZ "Z: " pos:[15,174] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanRotStepZ "" pos:[93,174] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanRotBothZ "" pos:[142,175] width:14 height:14

		ImgTag bmpBothWay2 "" pos:[142,195] width:13 height:14 bitmap:(openBitMap (pathDir + "array_icon_bothway.png")) transparent:white

		label lblStepSca "Step" pos:[100,196] width:23 height:14
		label lblScaling "Scaling" pos:[30,196] width:37 height:14
		spinner floatRanScaX "X: " pos:[15,210] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanScaStepX "" pos:[93,211] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanScaBothX "" pos:[142,212] width:14 height:14
		spinner floatRanScaY "Y: " pos:[15,230] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanScaStepY "" pos:[93,230] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanScaBothY "" pos:[142,230] width:14 height:14
		spinner floatRanScaZ "Z: " pos:[15,250] width:74 height:16 range:[-360,360,0] scale:0.1
		spinner floatRanScaStepZ "" pos:[93,250] width:44 height:16 range:[0.001,100000,1] scale:1
		checkbox boolRanScaBothZ "" pos:[142,250] width:14 height:14
		
		button btnManualUpdate "Manual Update" pos:[37,272] width:88 height:20
	
		on boolRandomizeOn changed theState do
		(
			intRanSeed.enabled = theState
			floatRanPosX.enabled = floatRanPosY.enabled = floatRanPosZ.enabled = theState
			boolRanPosBothX.enabled = boolRanPosBothY.enabled = boolRanPosBothZ.enabled = theState
			floatRanPosStepX.enabled = floatRanPosStepY.enabled = floatRanPosStepZ.enabled = theState
			floatRanRotX.enabled = floatRanRotY.enabled = floatRanRotZ.enabled = theState
			boolRanRotBothX.enabled = boolRanRotBothY.enabled = boolRanRotBothZ.enabled = theState
			floatRanRotStepX.enabled = floatRanRotStepY.enabled = floatRanRotStepZ.enabled = theState
			floatRanScaX.enabled = floatRanScaY.enabled = floatRanScaZ.enabled = theState
			boolRanScaBothX.enabled = boolRanScaBothY.enabled = boolRanScaBothZ.enabled = theState
			floatRanScaStepX.enabled = floatRanScaStepY.enabled = floatRanScaStepZ.enabled = theState
			
			updateRandom 1 1 1
		)
		on btnGenerateSeed pressed do
		(
			seed(random 1 9999999)
			intRanSeed.value = random 1 999999
			updateRandom 1 1 1
		)

		on floatRanPosX changed val do updateRandom 1 0 0
		on floatRanPosY changed val do updateRandom 1 0 0
		on floatRanPosZ changed val do updateRandom 1 0 0
		on floatRanRotX changed val do updateRandom 0 1 0
		on floatRanRotY changed val do updateRandom 0 1 0
		on floatRanRotZ changed val do updateRandom 0 1 0
		on floatRanScaX changed val do updateRandom 0 0 1
		on floatRanScaY changed val do updateRandom 0 0 1
		on floatRanScaZ changed val do updateRandom 0 0 1
		
		on floatRanPosStepX changed val do updateRandom 1 0 0
		on floatRanPosStepY changed val do updateRandom 1 0 0
		on floatRanPosStepZ changed val do updateRandom 1 0 0
		on floatRanRotStepX changed val do updateRandom 0 1 0
		on floatRanRotStepY changed val do updateRandom 0 1 0
		on floatRanRotStepZ changed val do updateRandom 0 1 0
		on floatRanScaStepX changed val do updateRandom 0 0 1
		on floatRanScaStepY changed val do updateRandom 0 0 1
		on floatRanScaStepZ changed val do updateRandom 0 0 1
		
		on boolRanPosBothX changed theState do updateRandom 1 0 0
		on boolRanPosBothY changed theState do updateRandom 1 0 0
		on boolRanPosBothZ changed theState do updateRandom 1 0 0
		on boolRanRotBothX changed theState do updateRandom 0 1 0
		on boolRanRotBothY changed theState do updateRandom 0 1 0
		on boolRanRotBothZ changed theState do updateRandom 0 1 0
		on boolRanScaBothX changed theState do updateRandom 0 0 1
		on boolRanScaBothY changed theState do updateRandom 0 0 1
		on boolRanScaBothZ changed theState do updateRandom 0 0 1
		
		on btnManualUpdate pressed do  updateRandom 1 1 1
		
		on rolloutRandomize open do
		(
			theState = boolRandomizeOn.state
			intRanSeed.enabled = theState
			floatRanPosX.enabled = floatRanPosY.enabled = floatRanPosZ.enabled = theState
			boolRanPosBothX.enabled = boolRanPosBothY.enabled = boolRanPosBothZ.enabled = theState
			floatRanPosStepX.enabled = floatRanPosStepY.enabled = floatRanPosStepZ.enabled = theState
			floatRanRotX.enabled = floatRanRotY.enabled = floatRanRotZ.enabled = theState
			boolRanRotBothX.enabled = boolRanRotBothY.enabled = boolRanRotBothZ.enabled = theState
			floatRanRotStepX.enabled = floatRanRotStepY.enabled = floatRanRotStepZ.enabled = theState
			floatRanScaX.enabled = floatRanScaY.enabled = floatRanScaZ.enabled = theState
			boolRanScaBothX.enabled = boolRanScaBothY.enabled = boolRanScaBothZ.enabled = theState
			floatRanScaStepX.enabled = floatRanScaStepY.enabled = floatRanScaStepZ.enabled = theState
			
		)
	)
	
	
	
)