-- =======================================================
-- == Updater for Array 3D modifier, now Array 3D Helper ==
-- == Created by Anders K. "McGreed" Nielsen                   ==
-- =======================================================
global updateArray3D, arrayObjects, arrModifier
try destroyDialog updateArray3D catch()


fn updateSceneObjects =
(
	with undo off
	(
		arrayObjects = #()
		arrModifier = #()
		sceneObjects = selection -- Get all objects selected
		if sceneObjects.count == 0 do -- If no objects selected, get all objects in scene
			sceneObjects = $*
	
		countObjects = 0
		countArray = 0
		countRandom = 0
		updateArray3D.lbxObjectList.items = #()
		for obj in sceneObjects do
		(
			for m in obj.modifiers do
			(
				modClass = classOf m as string
				if modClass ==  "Array3DMod" or modClass == "ArrayRandomizeMod" do
				(
					existingItem = findItem arrayObjects obj
					if existingItem == 0 then
					(
						countObjects += 1
						arrayObjects = append arrayObjects obj
						if modClass ==  "Array3DMod" then
							tempMod = (#(1,0))
						else
							tempMod = (#(0,1))

						arrModifier = append arrModifier tempMod
					) else (
						if modClass ==  "Array3DMod" then
							arrModifier[existingItem][1] += 1
						else
							arrModifier[existingItem][2] += 1
					)
				)	
		 		if (classOf m) as string == "Array3DMod" do
					countArray += 1
		 		if (classOf m) as string == "ArrayRandomizeMod" do
					countRandom += 1
		 	)
		)
		for i = 1 to arrayObjects.count do
		(
			tempText =  arrayObjects[i].name + " (" + arrModifier[i][1] as string + " / " + arrModifier[i][2] as string + ")"
			updateArray3D.lbxObjectList.items = append updateArray3D.lbxObjectList.items tempText
		)
		updateArray3D.lblSelectedObjectCount.text = "Selected Objects: " + countObjects as string
		updateArray3D.lblArrayCount.text = "Array3D: " + countArray as string
		updateArray3D.lblRandomCount.text = "Array Randomize: " + countRandom as string
	)
	arrModifier = sceneObjects = modClass = tempMod = tempText = undefined
)
fn removeFromList =
(
	with undo off
	(
		tempPrefix = strArrayPrefixName
		whichSelection = updateArray3D.lbxObjectList.selection
		whichItem = whichSelection as array
		for i in whichItem.count to 1 by -1 do
		(
			arrayObjects = deleteItem arrayObjects whichItem[i]
			updateArray3D.lbxObjectList.items = deleteItem updateArray3D.lbxObjectList.items whichItem[i]
		)
		tempName = undefined
	)
)
fn updateModifiers =
(
	with undo off
	(
		ProgressStart "Updating modifiers..."
		--disableSceneRedraw()
		totalFunctionCounts = 0
		colArray3DSingle = #()
		colArray3DMulti = #()
		colDelMulti = #()
		for o in arrayObjects do
		(
			totalFunctionCounts += (for modArray3D in o.modifiers where ((modArray3D.classid)[1] == 1864062083 or (modArray3D.classid)[1] == 2056880029) collect modArray3D).count
			
			countArray3D = (for modArray3D in o.modifiers where (modArray3D.classid)[1] == 1864062083 collect i).count
			-- Sort out the dummies with more then one Array 3D modifier assigned to it
			if countArray3D > 1 then
				colArray3DMulti = append colArray3DMulti o
			else
				colArray3DSingle = append colArray3DSingle o
		)
		-- Update dummies with multi array3d modifiers into single array dummies
		for o in colArray3DMulti do
		(
			for modArray3D in o.modifiers where (modArray3D.classid)[1] == 1864062083 do
			(
				newName = (uniquename(o.name as string) as string)
				newDummy = copy o name:newName
				
				oldID = modArray3D.intUniqueArrayID
				for delArray3D = newDummy.modifiers.count to 1 by -1 do
					if ((newDummy.modifiers[delArray3D].classid)[1] == 1864062083 or (newDummy.modifiers[delArray3D].classid)[1] == 2056880029) and newDummy.modifiers[delArray3D].intUniqueArrayID != oldID do
						deleteModifier newDummy delArray3D

				colArray3DSingle = append colArray3DSingle newDummy
				colDelMulti = append colDelMulti newDummy -- list of temporary dummies that needs to be deleted
			)
 			for d = o.modifiers.count to 1 by -1 do
 				if o.modifiers[d].classid[1] == 1864062083 or o.modifiers[d].classid[1] == 2056880029 do
 					deleteModifier o d
		)
		

		-- Prepare base objects
		baseObjects = #()
			

		for o in colArray3DSingle do
		(
			newName = "temporary_array3D_helper"
			newArrayHelper = Array3DHelper pos:[0,0,0] name:newName
			for modArray3D in o.modifiers where ((modArray3D.classid)[1] == 1864062083) do
			(
				oldArrayVersion = modArray3D.version
				newArrayHelper.transform = o.transform
				
				newArrayHelper.intUniqueArrayID = modArray3D.intUniqueArrayID
				newArrayHelper.intArrayType = modArray3D.intArrayType
				newArrayHelper.strSourceObjectList = modArray3D.strSourceObjectList
				newArrayHelper.strArrayPrefixName = modArray3D.strArrayPrefixName
				newArrayHelper.intDuplicateAction = modArray3D.intDuplicateAction
				newArrayHelper.intAAmount1D = modArray3D.intAAmount1D
				newArrayHelper.intAAmount2D = modArray3D.intAAmount2D
				newArrayHelper.intAAmount3D = modArray3D.intAAmount3D
			
				newArrayHelper.intShowAmount1D = modArray3D.intShowAmount1D
				newArrayHelper.intShowAmount2D = modArray3D.intShowAmount2D
				newArrayHelper.intShowAmount3D = modArray3D.intShowAmount3D
				newArrayHelper.intObjCreation = modArray3D.intObjCreation
				newArrayHelper.intWireColorFrom = modArray3D.intWireColorFrom
				newArrayHelper.colWirePalette = modArray3D.colWirePalette
				newArrayHelper.boolFreeze = modArray3D.boolFreeze
				newArrayHelper.boolFrozenAsGrey = false
				newArrayHelper.boolBoundingBox = modArray3D.boolBoundingBox

				newArrayHelper.boolPositionEnabled = modArray3D.boolPositionEnabled

				newArrayHelper.floatPosition1DX = modArray3D.floatPosition1DX
				newArrayHelper.floatPosition1DY = modArray3D.floatPosition1DY
				newArrayHelper.floatPosition1DZ = modArray3D.floatPosition1DZ
				newArrayHelper.floatPosition2DX = modArray3D.floatPosition2DX
				newArrayHelper.floatPosition2DY = modArray3D.floatPosition2DY
				newArrayHelper.floatPosition2DZ = modArray3D.floatPosition2DZ
				newArrayHelper.floatPosition3DX = modArray3D.floatPosition3DX
				newArrayHelper.floatPosition3DY = modArray3D.floatPosition3DY
				newArrayHelper.floatPosition3DZ = modArray3D.floatPosition3DZ
				newArrayHelper.boolPositionLink1D = modArray3D.boolPositionLink1D
				newArrayHelper.boolPositionLink2D = modArray3D.boolPositionLink2D
				newArrayHelper.boolPositionLink3D = modArray3D.boolPositionLink3D
				newArrayHelper.intSpacingType = modArray3D.intSpacingType
				
				newArrayHelper.strPos1DSpline = modArray3D.strPos1DSpline
				newArrayHelper.strPos2DSpline = modArray3D.strPos2DSpline
				newArrayHelper.strPos3DSpline = modArray3D.strPos3DSpline
				newArrayHelper.boolActivePos1DSpline = modArray3D.boolActivePos1DSpline
				newArrayHelper.boolActivePos2DSpline = modArray3D.boolActivePos2DSpline
				newArrayHelper.boolActivePos3DSpline = modArray3D.boolActivePos3DSpline
				newArrayHelper.intSplinePosSpread = modArray3D.intSplinePosSpread

				newArrayHelper.boolRotationEnabled = modArray3D.boolRotationEnabled

				newArrayHelper.floatRotation1DX = modArray3D.floatRotation1DX
				newArrayHelper.floatRotation1DY = modArray3D.floatRotation1DY
				newArrayHelper.floatRotation1DZ = modArray3D.floatRotation1DZ
				newArrayHelper.floatRotation2DX = modArray3D.floatRotation2DX
				newArrayHelper.floatRotation2DY = modArray3D.floatRotation2DY
				newArrayHelper.floatRotation2DZ = modArray3D.floatRotation2DZ
				newArrayHelper.floatRotation3DX = modArray3D.floatRotation3DX
				newArrayHelper.floatRotation3DY = modArray3D.floatRotation3DY
				newArrayHelper.floatRotation3DZ = modArray3D.floatRotation3DZ
				newArrayHelper.boolRotationLink1D = modArray3D.boolRotationLink1D
				newArrayHelper.boolRotationLink2D = modArray3D.boolRotationLink2D
				newArrayHelper.boolRotationLink3D = modArray3D.boolRotationLink3D
				newArrayHelper.intRotationType = modArray3D.intRotationType
				newArrayHelper.boolRotationObject = modArray3D.boolRotationEnabled
				newArrayHelper.boolSplineRotation = false
				newArrayHelper.boolIncrementRotation = false

				newArrayHelper.boolScalingEnabled = modArray3D.boolScalingEnabled

				newArrayHelper.floatScaling1DX = modArray3D.floatScaling1DX
				newArrayHelper.floatScaling1DY = modArray3D.floatScaling1DY
				newArrayHelper.floatScaling1DZ = modArray3D.floatScaling1DZ
				newArrayHelper.floatScaling2DX = modArray3D.floatScaling2DX
				newArrayHelper.floatScaling2DY = modArray3D.floatScaling2DY
				newArrayHelper.floatScaling2DZ = modArray3D.floatScaling2DZ
				newArrayHelper.floatScaling3DX = modArray3D.floatScaling3DX
				newArrayHelper.floatScaling3DY = modArray3D.floatScaling3DY
				newArrayHelper.floatScaling3DZ = modArray3D.floatScaling3DZ
				newArrayHelper.boolScalingLink1D = modArray3D.boolScalingLink1D
				newArrayHelper.boolScalingLink2D = modArray3D.boolScalingLink2D
				newArrayHelper.boolScalingLink3D = modArray3D.boolScalingLink3D
				
				if oldArrayVersion > 2 do -- If the old modifier is internal version above 2
					newArrayHelper.intScalingType = modArray3D.intScalingType

				if modArray3D.intLinkingTo == 1 and modArray3D.strLinkParent != o.name then
				( -- If the linking is to itself, and not the same as current dummy, relink
					linkingCheck = 1
					newArrayHelper.strLinkParent = o.name
				) else (
					newArrayHelper.strLinkParent = modArray3D.strLinkParent
					linkingCheck = 0
				)

				newArrayHelper.intLinkingTo = modArray3D.intLinkingTo
				newArrayHelper.strLinkCustom = modArray3D.strLinkCustom
				newArrayHelper.boolLinkingSplines = true

				newArrayHelper.floatOffsetPosX = modArray3D.floatOffsetPosX
				newArrayHelper.floatOffsetPosY = modArray3D.floatOffsetPosY
				newArrayHelper.floatOffsetPosZ = modArray3D.floatOffsetPosZ

				newArrayHelper.floatOffsetRotX = modArray3D.floatOffsetRotX
				newArrayHelper.floatOffsetRotY = modArray3D.floatOffsetRotY
				newArrayHelper.floatOffsetRotZ = modArray3D.floatOffsetRotZ

				newArrayHelper.intRanObjectSeed = modArray3D.intRanObjectSeed
				
				newArrayHelper.strConvertNewName = modArray3D.strConvertNewName
				newArrayHelper.strConvertCustom = modArray3D.strConvertCustom
				newArrayHelper.strConvertTo = modArray3D.strConvertTo
				newArrayHelper.strGroupName = modArray3D.strGroupName
				
				o.baseobject = newArrayHelper -- Replace old dummy with new helper
				o.wireColor = (color 87 225 87)
				
				if linkingCheck == 1 do
				( -- The array objects needs to be relinked to the helper
					tempExecuteList = #()
					for i in o.strSourceObjectList do -- Go though all the selected objects
					(
						if o.intArrayType == 2 then -- 1: Group objects , 2: Random objects
							tempExecuteList = append tempExecuteList (("$'" + (o.strArrayPrefixName as string) + "Ran" + o.intUniqueArrayID as string + ".arr_*x*x*'"))
						else (
							tempExecuteList = append tempExecuteList (("$'" + (o.strArrayPrefixName as string) + i + ".arr_*x*x*'"))
						)
						
					)
					
					for i in tempExecuteList do -- Go though all the selected objects
						execute (i + ".parent=$" + (o.strLinkParent as string))

					tempExecuteList = undefined -- release variable
				)

				deleteModifier o modArray3D -- Delete old Array 3D Modifier
				
				-- Get all old Array Randomize modifiers values and apply them to the new Array 3D Randomize modifiers
				newModifier = #()
				ranModifierCount = o.modifiers.count
				for m = 1 to ranModifierCount do
				(
					mar = o.modifiers[m]
					if (mar.classid)[1] == 2056880029 do
					(
						tempModifier = Array3DRandomizeMod ()
						addModifier o (tempModifier) -- Create new Array 3D Randomize modifier

						tempModifier.intRanSeed = mar.intRanSeed

						tempModifier.boolRandomizeOn = mar.boolRandomizeOn
						tempModifier.intRanSeed = mar.intRanSeed

						tempModifier.floatRanPosX = mar.floatRanPosX
						tempModifier.boolRanPosBothX = mar.boolRanPosBothX
						tempModifier.floatRanPosStepX = mar.floatRanPosStepX
						tempModifier.floatRanPosY = mar.floatRanPosY
						tempModifier.boolRanPosBothY = mar.boolRanPosBothY
						tempModifier.floatRanPosStepY = mar.floatRanPosStepY
						tempModifier.floatRanPosZ = mar.floatRanPosZ
						tempModifier.boolRanPosBothZ = mar.boolRanPosBothZ
						tempModifier.floatRanPosStepZ = mar.floatRanPosStepZ
								
						tempModifier.floatRanRotX = mar.floatRanRotX
						tempModifier.boolRanRotBothX = mar.boolRanRotBothX
						tempModifier.floatRanRotStepX = mar.floatRanRotStepX
						tempModifier.floatRanRotY = mar.floatRanRotY
						tempModifier.boolRanRotBothY = mar.boolRanRotBothY
						tempModifier.floatRanRotStepY = mar.floatRanRotStepY
						tempModifier.floatRanRotZ = mar.floatRanRotZ
						tempModifier.boolRanRotBothZ = mar.boolRanRotBothZ
						tempModifier.floatRanRotStepZ = mar.floatRanRotStepZ
						
						tempModifier.floatRanScaX = mar.floatRanScaX
						tempModifier.boolRanScaBothX = mar.boolRanScaBothX
						tempModifier.floatRanScaStepX = mar.floatRanScaStepX
						tempModifier.floatRanScaY = mar.floatRanScaY
						tempModifier.boolRanScaBothY = mar.boolRanScaBothY
						tempModifier.floatRanScaStepY = mar.floatRanScaStepY
						tempModifier.floatRanScaZ = mar.floatRanScaZ
						tempModifier.boolRanScaBothZ = mar.boolRanScaBothZ
						tempModifier.floatRanScaStepZ = mar.floatRanScaStepZ
						deleteModifier o (m+1) -- delete old randomize array from stack
					)
				)
			)
			delete newArrayHelper -- delete temporary helper
			
			tempModifier = ranModifierCount = undefined -- release variable
			
			
			-- Update the source in array
			for x = 1 to o.strSourceObjectList.count do
			(
				strSourceName = (o.strSourceObjectList[x] as string)
				strSourceObj = getNodeByName (strSourceName)
				tempArr = #()
				if strSourceObj == undefined then
				( --
					tempArr = append tempArr ("**MISSING** " + strSourceName as string)
					tempArr = append tempArr undefined
				) else (
					tempArr = append tempArr strSourceName
					tempArr = append tempArr strSourceObj
				)
				o.arrSourceDetails[x] = tempArr
				--arrSourceList.items = append arrSourceList.items strSourceName -- Add the source name to the list
			)
			strSourceName = strSourceObj = tempArr = undefined -- release variables
		)
		
		completeMessage = "Updating Complete!\n\nAll selected Array 3D and Array Randomize modifiers has been updated. If you don't have any more objects to update, shutdown 3DS Max and then delete the /array3d/ folder under /scripts/ to remove the old version."
		if colArray3DMulti.count > 0 do
			completeMessage += "\nNote: One or more dummies had several Array 3D modifiers assigned to them and has therefor been split between same amount of helpers and the original dummy has not been deleted."
		
		colArray3DSingle = colArray3DMulti = colDelMulti = undefined -- release variable
		
		updateSceneObjects ()
	)
	enableSceneRedraw()
	redrawViews()	
	progressend ()

	messageBox completeMessage
)

rollout updateArray3D "Update Array3D to Helper" width:460 height:335
(
	GroupBox grpArraySceneInfo "Array Scene Info" pos:[5,5] width:136 height:65
	label lblSelectedObjectCount "Selected Objects: 0" pos:[11,20] width:114 height:15		
	label lblArrayCount "Array 3D: 0" pos:[11,36] width:110 height:15
	label lblRandomCount "Array Randomize: 0" pos:[11,51] width:113 height:15
	GroupBox grpArray3DList "" pos:[6,70] width:135 height:222
	multiListBox lbxObjectList "Dummy/Array/Randomize" pos:[10,80] width:127 height:12
	button btnRefresh "Refresh" pos:[75,265] width:61 height:21
	button btnRemoveFromList "Remove" pos:[11,265] width:60 height:21
	button btnUpdate "Start Updating" pos:[15,300] width:111 height:25
	GroupBox grpUpdatetInfo "" pos:[147,5] width:304 height:320
	label lblUpdateInfo "Updater for Array 3D version 0.82+.
============================
	
Load a scene which contains objects with the old Array 3D modifier assigned to them. If you want to update all modifiers, make sure you have nothing selected and press the Refresh button and it will retieve all with the old modifier.
If you only want to update some dummies, just select the objects you want to update and press the Refresh button to add them to the list.

Please note that this updater will REPLACE the base object with the new Array 3D helper, so if you have assigned the modifier to an object you wish to preserve, you should copy it and remove the modfiers from the copy.

Now click the Start Updating button. After you are done, feel free to delete the old array version (shutdown 3DS Max and delete the /Array3D/ folder from /Scripts/ under 3DS Max). You can still use the old version alongside with the new version, but you will get two randomize modifiers in your list, the old Array Randomize and the new Array 3D Randomize.

" pos:[156,15] width:290 height:300

	on updateArray3D open do
		updateSceneObjects ()

	on updateArray3D close do
	(
		enableSceneRedraw()
		CompleteRedraw()
	)
	on btnRefresh pressed do
	(
		updateSceneObjects ()
	)
	on btnRemoveFromList pressed do
	(
		removeFromList ()
	)
	on btnUpdate pressed do
	(
		if lbxObjectList.items.count > 0 do
			updateModifiers ()
	)
)


createDialog updateArray3D 460 335 60 130
